﻿namespace ExtSpec.Build
{
    using System.IO;
    using System.Reflection;

    /// <summary>
    /// Contains the context used to build the library.
    /// </summary>
    public class BuildContext : IBuildContext
    {
        private const string OutputFileFormat = "{0}.js";

        private const string TypeScriptDefinitionFileFormat = "{0}.d.ts";

        private const string MainZipFormat = "{0}-{1}.zip";

        private const string ExampleZipFormat = "{0}-{1}-Examples.zip";

        private const string LicenseFileName = "License.txt";

        /// <summary>
        /// Initializes a new instance of the <see cref="BuildContext" /> class.
        /// </summary>
        /// <param name="assembly">The assembly.</param>
        /// <param name="rootPath">The root path.</param>
        public BuildContext(Assembly assembly, string rootPath)
        {
            this.Name = assembly.GetName().Name;
            this.Title = assembly.GetAttributeValue<AssemblyTitleAttribute>(x => x.Title);
            this.Version = assembly.GetAttributeValue<AssemblyInformationalVersionAttribute>(x => x.InformationalVersion);
            this.Copyright = assembly.GetAttributeValue<AssemblyCopyrightAttribute>(x => x.Copyright);

            this.RootPath = rootPath;
            this.BuildPath = Path.Combine(rootPath, "..", DirectoryName.Build);
            this.ExamplePath = Path.Combine(rootPath, DirectoryName.Example);
            this.LibraryPath = Path.Combine(rootPath, DirectoryName.Library);

            this.MainZipFilePath = Path.Combine(this.BuildPath, this.FormatFileName(MainZipFormat));
            this.ExampleZipFilePath = Path.Combine(this.BuildPath, this.FormatFileName(ExampleZipFormat));
            this.OutputFilePath = Path.Combine(this.BuildPath, this.FormatFileName(OutputFileFormat));
            this.TypeScriptDefinitionFilePath = Path.Combine(this.BuildPath, this.FormatFileName(TypeScriptDefinitionFileFormat));
            this.LicenseFilePath = Path.Combine(this.RootPath, LicenseFileName);
        }

        /// <summary>
        /// Gets the short name of the library.
        /// </summary>
        /// <value>
        /// The name.
        /// </value>
        public string Name { get; private set; }

        /// <summary>
        /// Gets the title of the library.
        /// </summary>
        /// <value>
        /// The title.
        /// </value>
        public string Title { get; private set; }

        /// <summary>
        /// Gets the version of the library.
        /// </summary>
        /// <value>
        /// The version.
        /// </value>
        public string Version { get; private set; }

        /// <summary>
        /// Gets the copyright statement of the library.
        /// </summary>
        /// <value>
        /// The copyright.
        /// </value>
        public string Copyright { get; private set; }

        /// <summary>
        /// Gets the root directory path.
        /// </summary>
        /// <value>
        /// The root path.
        /// </value>
        public string RootPath { get; private set; }

        /// <summary>
        /// Gets the build directory path.
        /// </summary>
        /// <value>
        /// The build path.
        /// </value>
        public string BuildPath { get; private set; }

        /// <summary>
        /// Gets the example directory path.
        /// </summary>
        /// <value>
        /// The example path.
        /// </value>
        public string ExamplePath { get; private set; }

        /// <summary>
        /// Gets the library directory path.
        /// </summary>
        /// <value>
        /// The library path.
        /// </value>
        public string LibraryPath { get; private set; }

        /// <summary>
        /// Gets the main zip file path.
        /// </summary>
        /// <value>
        /// The main zip file path.
        /// </value>
        public string MainZipFilePath { get; private set; }

        /// <summary>
        /// Gets the example zip file path.
        /// </summary>
        /// <value>
        /// The example zip file path.
        /// </value>
        public string ExampleZipFilePath { get; private set; }

        /// <summary>
        /// Gets the output file path.
        /// </summary>
        /// <value>
        /// The output file path.
        /// </value>
        public string OutputFilePath { get; private set; }

        /// <summary>
        /// Gets the type script definition file path.
        /// </summary>
        /// <value>
        /// The type script definition file path.
        /// </value>
        public string TypeScriptDefinitionFilePath { get; private set; }

        /// <summary>
        /// Gets the license file path.
        /// </summary>
        /// <value>
        /// The license file path.
        /// </value>
        public string LicenseFilePath { get; private set; }

        internal string FormatFileName(string fileNameFormat)
        {
            return string.Format(fileNameFormat, this.Name, this.Version);
        }
    }
}
