﻿namespace ExtSpec.Build
{
    using System.Reflection;
    using Autofac;

    /// <summary>
    /// Provides core building services.
    /// </summary>
    public class BuildProvider : IBuildProvider
    {
        private IFileSystemProvider fileSystemProvider;

        private IScriptCompiler scriptCompiler;

        private IZipProvider zipProvider;

        /// <summary>
        /// Initializes a new instance of the <see cref="BuildProvider" /> class.
        /// </summary>
        /// <param name="fileSystemProvider">The file system provider.</param>
        /// <param name="scriptCompiler">The script compiler.</param>
        /// <param name="zipProvider">The zip provider.</param>
        public BuildProvider(IFileSystemProvider fileSystemProvider, IScriptCompiler scriptCompiler, IZipProvider zipProvider)
        {
            this.fileSystemProvider = fileSystemProvider;
            this.scriptCompiler = scriptCompiler;
            this.zipProvider = zipProvider;
        }

        /// <summary>
        /// Registers a build context creates a provider based on the given assembly and root path.
        /// </summary>
        /// <param name="assembly">The assembly.</param>
        /// <param name="rootPath">The root path.</param>
        /// <returns>
        /// A new instance of the <see cref="BuildProvider" /> class.
        /// </returns>
        public static IBuildProvider Create(Assembly assembly, string rootPath)
        {
            var builder = new ContainerBuilder();

            builder
                .RegisterAssemblyTypes(typeof(BuildProvider).Assembly)
                .Where(x => x.GetInterface("I" + x.Name) != null)
                .Except<BuildContext>()
                .As(x => x.GetInterface("I" + x.Name));

            builder
                .RegisterInstance<IBuildContext>(new BuildContext(assembly, rootPath));

            var container = builder.Build();

            return container.Resolve<IBuildProvider>();
        }

        /// <summary>
        /// Builds the current context.
        /// </summary>
        public void Build()
        {
            this.fileSystemProvider.ResetBuildPath();

            this.scriptCompiler.Compile("Build.ts");
            this.scriptCompiler.PostProcess();

            this.fileSystemProvider.RebuildExampleLibrary();
            this.fileSystemProvider.CopyOutputToExamples();

            this.zipProvider.ZipExamples();
            this.zipProvider.ZipOutput();

            this.fileSystemProvider.CleanupBuildPath();
        }
    }
}
