﻿namespace ExtSpec.Build
{
    using System.IO;
    using System.Text.RegularExpressions;
    using ExtSpec.Build.Wrappers;

    /// <summary>
    /// Provides TypeScript compilation services.
    /// </summary>
    public class ScriptCompiler : BuildServiceBase, IScriptCompiler
    {
        private static readonly Regex HeaderPattern = new Regex(@"/\*![^!]*!\*/", RegexOptions.Compiled | RegexOptions.Multiline);

        private static readonly Regex ReferencePattern = new Regex(@"^ */// <reference[^\r\n]+(\r?\n|$)", RegexOptions.Compiled | RegexOptions.Multiline);

        private IProcessProvider processProvider;

        private IFileSystemWrapper fileSystemWrapper;

        /// <summary>
        /// Initializes a new instance of the <see cref="ScriptCompiler" /> class.
        /// </summary>
        /// <param name="context">The build context.</param>
        /// <param name="processProvider">The process provider.</param>
        /// <param name="fileSystemWrapper">The file system wrapper.</param>
        public ScriptCompiler(IBuildContext context, IProcessProvider processProvider, IFileSystemWrapper fileSystemWrapper)
            : base(context)
        {
            this.processProvider = processProvider;
            this.fileSystemWrapper = fileSystemWrapper;
        }

        /// <summary>
        /// Compiles the project.
        /// </summary>
        /// <param name="buildScript">The build script.</param>
        public void Compile(string buildScript)
        {
            var executable = Path.Combine(this.fileSystemWrapper.GetCurrentDirectory(), DirectoryName.TypeScript, "tsc.exe");
            var input = Path.Combine(this.Context.RootPath, buildScript);

            this.processProvider.Execute(executable, @"--target ES5 --declaration --out """ + this.Context.OutputFilePath + @""" """ + input + @"""");
        }

        /// <summary>
        /// Post processes the compiled library.
        /// </summary>
        public void PostProcess()
        {
            var contents = this.fileSystemWrapper.ReadAllText(this.Context.OutputFilePath);

            contents = HeaderPattern.Replace(contents, x => string.Format(x.Value, this.Context.Title, this.Context.Version, this.Context.Copyright));
            contents = ReferencePattern.Replace(contents, string.Empty);

            this.fileSystemWrapper.WriteAllText(this.Context.OutputFilePath, contents);
        }
    }
}
