﻿namespace ExtSpec.Build.Wrappers
{
    /// <summary>
    /// Wraps static file system operations.
    /// </summary>
    public interface IFileSystemWrapper
    {
        /// <summary>
        /// Creates all directories and subdirectories as specified by path.
        /// </summary>
        /// <param name="path">The directory path to create.</param>
        void CreateDirectory(string path);

        /// <summary>
        /// Copies the source directory to the destination path, creating it if it does not exist.
        /// </summary>
        /// <param name="source">The source directory path.</param>
        /// <param name="destination">The destination directory path.</param>
        void CopyDirectory(string source, string destination);

        /// <summary>
        /// Empties the directory of all files and subdirectories.
        /// </summary>
        /// <param name="path">The directory path to empty.</param>
        void EmptyDirectory(string path);

        /// <summary>
        /// Deletes the specified directory and, if indicated, any subdirectories in the directory.
        /// </summary>
        /// <param name="path">The name of the directory to remove.</param>
        /// <param name="recursive"><c>true</c> to remove directories, subdirectories, and files in path; otherwise, <c>false</c>.</param>
        void DeleteDirectory(string path, bool recursive);

        /// <summary>
        /// Determines whether the given path refers to an existing directory on disk.
        /// </summary>
        /// <param name="path">The path to test.</param>
        /// <returns>
        /// <c>true</c> if path refers to an existing directory; otherwise, <c>false</c>.
        /// </returns>
        bool DirectoryExists(string path);

        /// <summary>
        /// Gets the current working directory of the application.
        /// </summary>
        /// <returns>
        /// A string that contains the path of the current working directory, and does not end with a backslash (\).
        /// </returns>
        string GetCurrentDirectory();

        /// <summary>
        /// Creates a new file, writes the specified string to the file, and then closes the file. If the target file already exists, it is overwritten.
        /// </summary>
        /// <param name="path">The file to write to.</param>
        /// <param name="contents">The string to write to the file.</param>
        void WriteAllText(string path, string contents);

        /// <summary>
        /// Opens a text file, reads all lines of the file, and then closes the file.
        /// </summary>
        /// <param name="path">The file to open for reading.</param>
        /// <returns>
        /// A string containing all lines of the file.
        /// </returns>
        string ReadAllText(string path);

        /// <summary>
        /// Copies an existing file to a new file. Overwriting a file of the same name is allowed.
        /// </summary>
        /// <param name="sourceFileName">The file to copy.</param>
        /// <param name="destFileName">The name of the destination file. This cannot be a directory.</param>
        /// <param name="overwrite">true if the destination file can be overwritten; otherwise, false.</param>
        void CopyFile(string sourceFileName, string destFileName, bool overwrite);

        /// <summary>
        /// Deletes the specified file.
        /// </summary>
        /// <param name="path">The name of the file to be deleted. Wildcard characters are not supported.</param>
        void DeleteFile(string path);
    }
}
