﻿namespace ExtSpec.Specifications.Build
{
    using ExtSpec.Build;
    using ExtSpec.Build.Wrappers;
    using Moq;
    using Xunit;

    public class FileSystemProviderUnit
    {
        public abstract class FileSystemProviderUnitBase : BuildServiceUnitBase<FileSystemProvider>
        {
            public FileSystemProviderUnitBase()
            {
                this.FileSystemWrapperMock = this.AutoMocker.Mock<IFileSystemWrapper>();
            }

            public Mock<IFileSystemWrapper> FileSystemWrapperMock { get; set; }
        }

        public class ResetBuildPath : FileSystemProviderUnitBase
        {
            [Fact]
            public void Should_empty_build_directory_if_it_exists()
            {
                this.FileSystemWrapperMock.Setup(x => x.DirectoryExists("BUILD PATH")).Returns(true);

                this.Instance.ResetBuildPath();

                this.FileSystemWrapperMock.Verify(x => x.EmptyDirectory("BUILD PATH"));
                this.FileSystemWrapperMock.Verify(x => x.CreateDirectory("BUILD PATH"), Times.Never());
            }

            [Fact]
            public void Should_create_build_directory_if_it_does_not_exist()
            {
                this.Instance.ResetBuildPath();

                this.FileSystemWrapperMock.Verify(x => x.CreateDirectory("BUILD PATH"));
                this.FileSystemWrapperMock.Verify(x => x.EmptyDirectory("BUILD PATH"), Times.Never());
            }
        }

        public class CleanupBuildPath : FileSystemProviderUnitBase
        {
            public CleanupBuildPath()
            {
                this.Instance.CleanupBuildPath();
            }

            [Fact]
            public void Should_remove_output_file()
            {
                this.FileSystemWrapperMock.Verify(x => x.DeleteFile(@"BUILD PATH\EXTSPEC.JS"));
            }

            [Fact]
            public void Should_remove_definition_file()
            {
                this.FileSystemWrapperMock.Verify(x => x.DeleteFile(@"BUILD PATH\EXTSPEC.D.TS"));
            }
        }

        public class RebuildExampleLibrary : FileSystemProviderUnitBase
        {
            private const string ExampleLibrary = @"EXAMPLE PATH\libraries";

            [Fact]
            public void Should_delete_example_library_if_it_exists()
            {
                this.FileSystemWrapperMock.Setup(x => x.DirectoryExists(ExampleLibrary)).Returns(true);

                this.Instance.RebuildExampleLibrary();

                this.FileSystemWrapperMock.Verify(x => x.DeleteDirectory(ExampleLibrary, true));
            }

            [Fact]
            public void Should_always_copy_library_directory()
            {
                this.Instance.RebuildExampleLibrary();

                this.FileSystemWrapperMock.Verify(x => x.CopyDirectory("LIBRARY PATH", ExampleLibrary));
                this.FileSystemWrapperMock.Verify(x => x.DeleteDirectory(ExampleLibrary, true), Times.Never());
            }
        }

        public class CopyOutputToExamples : FileSystemProviderUnitBase
        {
            private const string DistributionPath = @"EXAMPLE PATH\libraries\extspec";

            private const string ExampleLibraryPath = @"EXAMPLE PATH\libraries\extspec\EXTSPEC.JS";

            public CopyOutputToExamples()
            {
                this.Instance.CopyOutputToExamples();
            }

            [Fact]
            public void Should_create_distribution_directory_if_it_does_not_exist()
            {
                this.FileSystemWrapperMock.Verify(x => x.CreateDirectory(DistributionPath));
            }

            [Fact]
            public void Should_copy_output_file_to_examples_library()
            {
                this.FileSystemWrapperMock.Verify(x => x.CopyFile(It.IsAny<string>(), ExampleLibraryPath, true));
            }
        }
    }
}
