﻿namespace ExtSpec.Specifications.Build
{
    using ExtSpec.Build;
    using ExtSpec.Build.Wrappers;
    using ExtSpec.Specifications.Expectations;
    using Moq;
    using Xunit;

    public class ScriptCompilerUnit
    {
        public abstract class ScriptCompilerUnitBase : BuildServiceUnitBase<ScriptCompiler>
        {
            public ScriptCompilerUnitBase()
            {
                this.FileSystemWrapperMock = this.AutoMocker.Mock<IFileSystemWrapper>();
                this.ProcessProviderMock = this.AutoMocker.Mock<IProcessProvider>();
            }

            public Mock<IFileSystemWrapper> FileSystemWrapperMock { get; set; }

            public Mock<IProcessProvider> ProcessProviderMock { get; set; }
        }

        public class Compile : ScriptCompilerUnitBase
        {
            private string actualArguments;

            public Compile()
            {
                this.FileSystemWrapperMock
                    .Setup(x => x.GetCurrentDirectory()).Returns("CURRENT");

                this.ProcessProviderMock
                    .Setup(x => x.Execute(It.IsAny<string>(), It.IsAny<string>()))
                    .Callback((string x, string y) => this.actualArguments = y);

                this.Instance.Compile("BUILD.TS");
            }

            [Fact]
            public void Should_execute_typescript_compiler()
            {
                this.ProcessProviderMock.Verify(x => x.Execute(@"CURRENT\TypeScript\tsc.exe", It.IsAny<string>()));
            }

            [Fact]
            public void Should_compile_with_comments()
            {
                I.Expect(this.actualArguments).Not.ToContain("--removeComments");
            }

            [Fact]
            public void Should_compile_with_declaration_file()
            {
                I.Expect(this.actualArguments).ToContain("--declaration");
            }

            [Fact]
            public void Should_compile_with_correct_output()
            {
                I.Expect(this.actualArguments).ToContain(@"--out ""BUILD PATH\EXTSPEC.JS""");
            }

            [Fact]
            public void Should_compile_with_correct_input()
            {
                I.Expect(this.actualArguments).ToEndWith(@"""ROOT PATH\BUILD.TS""");
            }
        }

        public class PostProcess : ScriptCompilerUnitBase
        {
            private string actualContent;

            public PostProcess()
            {
                var returns = @"
/*!
{0}-{1}-{2}
http://extspec.codeplex.com/
!*/

    /// <reference one />

FILE CONTENTS

/// <reference two />";

                this.FileSystemWrapperMock
                    .Setup(x => x.ReadAllText(It.IsAny<string>()))
                    .Returns(returns);

                this.FileSystemWrapperMock
                    .Setup(x => x.WriteAllText(It.IsAny<string>(), It.IsAny<string>()))
                    .Callback((string x, string y) => this.actualContent = y);

                this.Instance.PostProcess();
            }

            [Fact]
            public void Should_inject_copyright_information()
            {
                var expected = @"
/*!
TITLE-1.2.3-COPYRIGHT
http://extspec.codeplex.com/
!*/";

                I.Expect(this.actualContent).ToStartWith(expected);
            }

            [Fact]
            public void Should_remove_references()
            {
                I.Expect(this.actualContent).Not.ToContain(@"/// <reference");
            }
        }
    }
}
