/// <reference path="Libraries/jasmine/jasmine.d.ts" />
/// <reference path="Core.ts" />

/**
 * Obsolete matchers for Jasmine.
 */
module ExtSpec.Jasmine.Matchers {
    'use strict';

    var bindRe: RegExp = /(?:^|\.)(?:on|addListener|mon|addManagedListener)$/,
        unbindRe: RegExp = /(?:^|\.)(?:un|removeListener|mun|removeMangedListener)$/,
        managedRe: RegExp = /(?:^|\.)(?:mon|addManagedListener|mun|removeMangedListener)$/;

    /**
     * Sets the message for a jasmine expectation.
     * @param {Object} expectation The expectation.
     * @param {String} prefix The start of the message.
     * @param {String} suffix The end of the message.
     */
    function setExpectationMessage(expectation: jasmine.Matchers, prefix: string, suffix: string): void {
        var message: string = prefix + (expectation.isNot ? ' not ' : ' ') + suffix;

        expectation.message = function () {
            return message;
        };
    }

    function eventBindingsMatch(expectation: jasmine.Matchers, bind: boolean, event: string, actual: Function, expected: Function, scope: any): boolean;
    function eventBindingsMatch(expectation: jasmine.Matchers, bind: boolean, event: string, actual: string, expected: string, scope: any): boolean;
    /**
     * Passes the expectation if the actual function matches the expected function.
     * @param {Object} expectation The expectation.
     * @param {Boolean} bind True if the function handles event binding, false for unbinding.
     * @param {String} event The event name.
     * @param {String|Function} actual The actual listener function or its name.
     * @param {String|Function} expected The expected listener function or its name.
     * @param {Object} [scope] The scope if provided.
     * @returns {Boolean}
     */
    function eventBindingsMatch(expectation: jasmine.Matchers, bind: boolean, event: string, actual: any, expected: any, scope: any): boolean {
        if (ExtSpec.isTypeOf(expected, 'fn') && ExtSpec.isTypeOf(actual, 'st')) {
            actual = scope[actual];
        }

        setExpectationMessage(
            expectation,
            'Expected ' + jasmine.pp(event) + ' event with ' + (bind ? 'binding to ' : 'unbinding from ') + jasmine.pp(actual),
            'to have been ' + (bind ? 'bound to ' : 'unbound from ') + jasmine.pp(expected) + '.'
        );

        return actual === expected;
    }

    function hasBoundEvent(expectation: jasmine.Matchers, bind: boolean, event: string, expected: Function, call?: number): boolean;
    function hasBoundEvent(expectation: jasmine.Matchers, bind: boolean, event: string, expected: string, call?: number): boolean;
    /**
     * Passes the expectation if the event binding matches the expected listener.
     * @param {Object} expectation The expectation.
     * @param {Boolean} bind True if the method handles event binding, false if it handles unbinding.
     * @param {String} event The event name.
     * @param {String|Function} expected The expected listener function or its name.
     * @param {Number} [call] The index of the spy call to analyze, or undefined to analyze the most recent call.
     * @returns {Boolean}
     */
    function hasBoundEvent(expectation: jasmine.Matchers, bind: boolean, event: string, expected: any, call?: number): boolean {
        if (!(expectation.actual && expectation.actual.isSpy)) {
            throw new Error('Expected ' + jasmine.pp(expectation.actual) + ' to be a spy.');
        }

        var spyName: string = expectation.actual.identity,
            allowedMethods: RegExp = bind ? bindRe : unbindRe,
            actual: any,
            args: any,
            managed: number,
            eventName: string,
            options: any,
            config: any;

        if (!allowedMethods.test(spyName)) {
            throw new Error('Expected ' + jasmine.pp(expectation.actual) + ' to be an event ' + (bind ? 'binder.' : 'unbinder.'));
        }

        actual = ExtSpec.isTypeOf(call, 'nm') ? expectation.actual.calls[call] : expectation.actual.mostRecentCall;
        args = actual.args;
        managed = managedRe.test(spyName) ? 1 : 0;
        eventName = args[managed];

        if (!ExtSpec.isTypeOf(eventName, 'st')) {
            options = eventName;

            if (options.hasOwnProperty(event)) {
                config = options[event];

                return eventBindingsMatch(
                    expectation,
                    bind,
                    event,
                    config.fn || config,
                    expected,
                    config.scope || options.scope || actual.object
                );
            }
        } else {
            if (eventName === event) {
                return eventBindingsMatch(
                    expectation,
                    bind,
                    event,
                    args[1 + managed],
                    expected,
                    args[2 + managed] || actual.object
                );
            }
        }

        setExpectationMessage(
            expectation,
            'Expected event ' + jasmine.pp(event) + ' with no ' + (bind ? 'binding' : 'unbinding'),
            'to have been ' + (bind ? 'bound to ' : 'unbound from ') + jasmine.pp(expected) + '.'
        );
        return false;
    }

    export function toHaveBoundEvent(event: string, expected: Function, call?: number): boolean;
    export function toHaveBoundEvent(event: string, expected: string, call?: number): boolean;
    /**
     * Passes if the spy bound the given event name to the expected listener.
     * @param {String} event The name of the event.
     * @param {Function|String} expected The listener function or its name to match.
     * @param {Number} [call] The index of the spy call to analyze, or undefined to analyze the most recent call.
     * @returns {Boolean}
     * @deprecated Use toHaveAddedListener or toHaveAddedManagedListener.
     */
    export function toHaveBoundEvent(event: string, expected: any, call?: number): boolean {
        ExtSpec.obsolete({
            namespace: 'ExtSpec.Jasmine.Matchers',
            method: 'toHaveBoundEvent',
            message: 'Use toHaveAddedListener or toHaveAddedManagedListener.'
        });

        return hasBoundEvent(this, true, event, expected, call);
    }

    export function toHaveUnboundEvent(event: string, expected: Function, call?: number): boolean;
    export function toHaveUnboundEvent(event: string, expected: string, call?: number): boolean;
    /**
     * Passes if the spy unbound the given event name from the expected listener.
     * @param {String} event The name of the event.
     * @param {Function|String} expected The listener function or its name to match.
     * @param {Number} [call] The index of the spy call to analyze, or undefined to analyze the most recent call.
     * @returns {Boolean}
     * @deprecated Use toHaveRemovedListener or toHaveRemovedManagedListener.
     */
    export function toHaveUnboundEvent(event: string, expected: any, call?: number): boolean {
        ExtSpec.obsolete({
            namespace: 'ExtSpec.Jasmine.Matchers',
            method: 'toHaveUnboundEvent',
            message: 'Use toHaveRemovedListener or toHaveRemovedManagedListener.'
        });

        return hasBoundEvent(this, false, event, expected, call);
    }

    export function toHaveControlledEvent(event: string, expected: Function, selector?: string, call?: number): boolean;
    export function toHaveControlledEvent(event: string, expected: string, selector?: string, call?: number): boolean;
    /**
     * Passes if the spy controlled the given event name using the expected listener.
     * @param {String} event The name of the event.
     * @param {Function|String} expected The listener function or its name to match.
     * @param {String} [selector] The optional component query selector to match.
     * @param {Number} [call] The index of the spy call to analyze, or undefined to analyze the most recent call.
     * @returns {Boolean}
     * @deprecated Use toHaveControlled.
     */
    export function toHaveControlledEvent(event: string, expected: any, selector?: string, call?: number): boolean {
        ExtSpec.obsolete({
            namespace: 'ExtSpec.Jasmine.Matchers',
            method: 'toHaveControlledEvent',
            message: 'Use toHaveControlled.'
        });

        if (!(this.actual && this.actual.isSpy)) {
            throw new Error('Expected ' + jasmine.pp(this.actual) + ' to be a spy.');
        }

        if (this.actual.identity !== 'control') {
            throw new Error('Expected ' + jasmine.pp(this.actual) + ' to be an event controller.');
        }

        var actual: jasmine.Call,
            selectors: any,
            options: any,
            key: string,
            config: any,
            match: boolean;

        actual = ExtSpec.isTypeOf(call, 'nm') ? this.actual.calls[call] : this.actual.mostRecentCall;
        selectors = actual.args[0];

        if (ExtSpec.isTypeOf(selectors, 'st')) {
            key = selectors;
            selectors = {};
            selectors[key] = actual.args[1];
        }

        if (selector) {
            if (selectors.hasOwnProperty(selector)) {
                options = selectors[selector];

                if (options.hasOwnProperty(event)) {
                    config = options[event];

                    return eventBindingsMatch(
                        this,
                        true,
                        event,
                        config.fn || config,
                        expected,
                        config.scope || options.scope || actual.object
                    );
                }
            }
        } else {
            for (key in selectors) {
                if (selectors.hasOwnProperty(key)) {
                    options = selectors[key];

                    if (options.hasOwnProperty(event)) {
                        config = options[event];
                        match = eventBindingsMatch(
                            this,
                            true,
                            event,
                            config.fn || config,
                            expected,
                            config.scope || options.scope || actual.object
                        );

                        if (match) {
                            return true;
                        }
                    }
                }
            }
        }

        setExpectationMessage(
            this,
            'Expected ' + jasmine.pp(event) + ' event' + (selector ? ' using ' + jasmine.pp(selector) + ' selector' : ''),
            'to have been controlled with ' + jasmine.pp(expected) + '.'
        );
        return false;
    }
}