/// <reference path="Libraries/jasmine/jasmine.d.ts" />
/// <reference path="Core.ts" />

/**
 * Matchers for Jasmine.
 */
module ExtSpec.Jasmine.Matchers {
    'use strict';

    /**
     * The configuration structure for a listened expectation.
     */
    export interface IListenedConfig {
        /**
         * The event name to match.
         */
        event: string;
        /**
         * The listener to match.
         */
        listener?: Function;
    }

    /**
     * The configuration structure for a controlled expectation.
     */
    export interface IControlledConfig extends IListenedConfig {
        /**
         * The string or regular expression selector to match.
         */
        selector?: any;
    }

    interface IListenedMatcherConfig {
        actual: any;
        expected: any;
        isManaged: boolean;
    }

    /**
     * Test if the expectation's actual value contains a name of the given type, and sets an appropriate message if it is not found.
     * @param {jasmine.ExpectationResult} expectation The expectation to test.
     * @param {String} name The name to find in the expectation's actual value.
     * @param {String} type A single character representing the type to find.
     * @returns {Boolean}
     */
    function hasItem(expectation: any, name: string, type: string): boolean {
        var config: ResourceConfig = ExtSpec.getResourceConfig(type),
            list: Array = expectation.actual && expectation.actual[config.property];

        return config.matcher(list, name);
    }

    /**
     * Tests if the expectations's actual value extends or overrides a specific class.
     * @param {jasmine.ExpectationResult} expectation The expectation to test.
     * @param {String} name The class name to match against the actual value.
     * @param {String} property The property name to match against the name.
     * @returns {Boolean}
     */
    function isExtension(expectation: jasmine.Matchers, name: string, property: string): boolean {
        var actual: any = expectation.actual;

        if (ExtSpec.isTypeOf(actual, 'fn')) {
            actual = actual.prototype;
        }

        return actual && actual[property] === name;
    }

    /**
     * Tests if a spy's arguments match the provided event configuration.
     * @param {Object} config The event configuration.
     * @returns {Boolean}
     */
    function listenedMatcher(config: IListenedMatcherConfig): boolean {
        if (!(config.actual && config.actual.isSpy)) {
            throw new Error('Expected ' + jasmine.pp(config.actual) + ' to be a spy.');
        }

        var actual: jasmine.Spy = config.actual,
            key: string,
            eventMap: any,
            listener: any,
            expected: IListenedConfig,
            i: number,
            len: number,
            args: any[],
            arg: number = config.isManaged ? 1 : 0;

        if (!ExtSpec.isTypeOf(config.expected, 'st')) {
            expected = config.expected;
        } else {
            expected = {
                event: config.expected
            };
        }

        for (i = 0, len = actual.calls.length; i < len; i += 1) {
            args = actual.calls[i].args;
            eventMap = args[arg];

            if (ExtSpec.isTypeOf(eventMap, 'st')) {
                key = eventMap;
                eventMap = {};
                eventMap[key] = args[arg + 1];
            }

            if (eventMap.hasOwnProperty(expected.event)) {
                if (!expected.listener) {
                    return true;
                }

                listener = eventMap[expected.event];

                if ((listener.fn || listener) === expected.listener) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Passes the expectation if the given model name is found.
     * @param {String} modelName The model name to find.
     * @returns {Boolean}
     */
    export function toHaveModel(modelName: string): boolean {
        return hasItem(this, modelName, 'md');
    }

    /**
     * Passes if the given view name is found.
     * @param {String} viewName The view name to find.
     * @returns {Boolean}
     */
    export function toHaveView(viewName: string): boolean {
        return hasItem(this, viewName, 'vw');
    }

    /**
     * Passes if the given controller name is found.
     * @param {String} controllerName The controller name to find.
     * @returns {Boolean}
     */
    export function toHaveController(controllerName: string): boolean {
        return hasItem(this, controllerName, 'ct');
    }

    /**
     * Passes if the given store name is found.
     * @param {String} storeName The store name to find.
     * @returns {Boolean}
     */
    export function toHaveStore(storeName: string): boolean {
        return hasItem(this, storeName, 'st');
    }

    /**
     * Passes if the given ref name is found.
     * @param {String} refName The ref name to find.
     * @returns {Boolean}
     */
    export function toHaveRef(refName: string): boolean {
        return hasItem(this, refName, 'rf');
    }

    /**
     * Passes if the given config name is found.
     * @param {String} configName The config name to find.
     * @returns {Boolean}
     */
    export function toHaveConfig(configName: string): boolean {
        return hasItem(this, configName, 'cf');
    }

    /**
     * Passes if the given class name is required.
     * @param {String} refName The required class name to find.
     * @returns {Boolean}
     */
    export function toRequire(className: string): boolean {
        return hasItem(this, className, 'rq');
    }

    /**
     * Passes if the class instance or constructor extends the given class name.
     * @param {String} className The class name to match.
     * @returns {Boolean}
     */
    export function toExtend(className: string): boolean {
        return isExtension(this, className, 'extend');
    }

    /**
     * Passes if the class instance or constructor overrides the given class name.
     * @param {String} className The class name to match.
     * @returns {Boolean}
     */
    export function toOverride(className: string): boolean {
        return isExtension(this, className, 'override');
    }

    /**
     * Passes if the keys in the expected object have matching values in a spy argument.
     * @param {Object} expected The object containing keys and values to match.
     * @param {Number} [arg] The zero based argument index to compare. Defaults to zero (the first).
     * @returns {Boolean}
     */
    export function toHaveBeenCalledWithConfig(expected: any, arg?: number): boolean {
        var i: number,
            len: number,
            spy: any = this.actual,
            config: any;

        if (!ExtSpec.isTypeOf(arg, 'nm')) {
            arg = 0;
        }

        if (spy && spy.isSpy) {
            for (i = 0, len = spy.calls.length; i < len; i += 1) {
                config = spy.calls[i].args[arg];

                if (config && ExtSpec.objectKeysMatch(config, expected)) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Passes if the spy added a listener for the given event name.
     * @param {String} expected The event name to match.
     * @returns {Boolean}
     */
    export function toHaveAddedListener(expected: string): boolean;
    /**
     * Passes if the spy added a listener for the given event configuration.
     * @param {Object} expected The event configuration to match.
     * @returns {Boolean}
     */
    export function toHaveAddedListener(expected: IListenedConfig): boolean;
    /**
     * Passes if the spy added a listener for the given event.
     * @param {String|Object} expected The event name or configuration to match.
     * @returns {Boolean}
     */
    export function toHaveAddedListener(expected: any): boolean {
        return listenedMatcher({
            actual: this.actual,
            expected: expected,
            isManaged: false
        });
    }

    /**
     * Passes if the spy removed a listener for the given event name.
     * @param {String} expected The event name to match.
     * @returns {Boolean}
     */
    export function toHaveRemovedListener(expected: string): boolean;
    /**
     * Passes if the spy removed a listener for the given event configuration.
     * @param {Object} expected The event configuration to match.
     * @returns {Boolean}
     */
    export function toHaveRemovedListener(expected: IListenedConfig): boolean;
    /**
     * Passes if the spy removed a listener for the given event.
     * @param {String|Object} expected The event name or configuration to match.
     * @returns {Boolean}
     */
    export function toHaveRemovedListener(expected: any): boolean {
        return listenedMatcher({
            actual: this.actual,
            expected: expected,
            isManaged: false
        });
    }

    /**
     * Passes if the spy added a listener for the given event name.
     * @param {String} expected The event name to match.
     * @returns {Boolean}
     */
    export function toHaveAddedManagedListener(expected: string): boolean;
    /**
     * Passes if the spy added a listener for the given event configuration.
     * @param {Object} expected The event configuration to match.
     * @returns {Boolean}
     */
    export function toHaveAddedManagedListener(expected: IListenedConfig): boolean;
    /**
     * Passes if the spy added a listener for the given event.
     * @param {String|Object} expected The event name or configuration to match.
     * @returns {Boolean}
     */
    export function toHaveAddedManagedListener(expected: any): boolean {
        return listenedMatcher({
            actual: this.actual,
            expected: expected,
            isManaged: true
        });
    }

    /**
     * Passes if the spy removed a listener for the given event name.
     * @param {String} expected The event name to match.
     * @returns {Boolean}
     */
    export function toHaveRemovedManagedListener(expected: string): boolean;
    /**
     * Passes if the spy removed a listener for the given event configuration.
     * @param {Object} expected The event configuration to match.
     * @returns {Boolean}
     */
    export function toHaveRemovedManagedListener(expected: IListenedConfig): boolean;
    /**
     * Passes if the spy removed a listener for the given event.
     * @param {String|Object} expected The event name or configuration to match.
     * @returns {Boolean}
     */
    export function toHaveRemovedManagedListener(expected: any): boolean {
        return listenedMatcher({
            actual: this.actual,
            expected: expected,
            isManaged: true
        });
    }

    /**
     * Passes if the spy controlled the given event name.
     * @param {String} expected The event name to match.
     * @returns {Boolean}
     */
    export function toHaveControlled(expected: string): boolean;
    /**
     * Passes if the spy controlled the given event configuration.
     * @param {Object} expected The configuration to match.
     * @returns {Boolean}
     */
    export function toHaveControlled(expected: IControlledConfig): boolean;
    /**
     * Passes if the spy controlled the given event.
     * @param {String|Object} expected The event name or configuration to match.
     * @returns {Boolean}
     */
    export function toHaveControlled(expected: any): boolean {
        if (!(this.actual && this.actual.isSpy)) {
            throw new Error('Expected ' + jasmine.pp(this.actual) + ' to be a spy.');
        }

        var actual: jasmine.Spy = this.actual,
            selector: string,
            selectorMap: any,
            selectorPattern: RegExp,
            args: any[],
            config: IControlledConfig,
            listener: any,
            i: number,
            len: number;

        if (!ExtSpec.isTypeOf(expected, 'st')) {
            config = expected;
        } else {
            config = {
                event: expected
            };
        }

        if (ExtSpec.isTypeOf(config.selector, 'rg')) {
            selectorPattern = config.selector;
        } else {
            selectorPattern = new RegExp(ExtSpec.regExpEscape(config.selector || ''));
        }

        for (i = 0, len = actual.calls.length; i < len; i += 1) {
            args = actual.calls[i].args;
            selectorMap = args[0];

            if (ExtSpec.isTypeOf(selectorMap, 'st')) {
                selector = selectorMap;
                selectorMap = {};
                selectorMap[selector] = args[1];
            }

            for (selector in selectorMap) {
                if (selectorMap.hasOwnProperty(selector) && selectorPattern.test(selector) && selectorMap[selector].hasOwnProperty(config.event)) {
                    if (!config.listener) {
                        return true;
                    }

                    listener = selectorMap[selector][config.event];

                    if ((listener.fn || listener) === config.listener) {
                        return true;
                    }
                }
            }
        }

        return false;
    }
}