﻿/// <reference path="../Libraries/jasmine/jasmine.d.ts" />
/// <reference path="../Jasmine.Matchers.ts" />
/// <reference path="../Jasmine.Matchers.obsolete.ts" />

declare module jasmine {
    interface Matchers {
        toHaveModel(modelName: string): boolean;
        toHaveView(viewName: string): boolean;
        toHaveController(controllerName: string): boolean;
        toHaveStore(storeName: string): boolean;
        toHaveRef(refName: string): boolean;
        toHaveConfig(configName: string): boolean;
        toRequire(className: string): boolean;
        toExtend(className: string): boolean;
        toOverride(className: string): boolean;
        toHaveBoundEvent(event: string, expected: any, call?: number): boolean;
        toHaveUnboundEvent(event: string, expected: any, call?: number): boolean;
        toHaveControlledEvent(event: string, expected: any, selector?: string, call?: number): boolean;
        toHaveBeenCalledWithConfig(expected: any, arg?: number): boolean;
        toHaveAddedListener(expected: any): boolean;
        toHaveRemovedListener(expected: any): boolean;
    }
}

describe('ExtSpec.Jasmine.Matchers', function () {
    'use strict';

    beforeEach(function () {
        this.addMatchers(ExtSpec.Jasmine.Matchers);
    });

    describe('toRequire', function () {
        it('should pass given a matching requirement', function () {
            var instance = {
                    requires: [
                        'Any.class.Name'
                    ]
                };

            expect(instance).toRequire('Any.class.Name');
        });

        it('should fail given a missing requirement', function () {
            var instance = {
                    requires: []
                };

            expect(instance).not.toRequire('Any.other.Name');
        });
    });

    describe('toHaveConfig', function () {
        it('should pass given an instance with a matching config property', function () {
            var instance = {
                    config: {
                        myProp: 22
                    }
                };

            expect(instance).toHaveConfig('myProp');
        });

        it('should fail given an instance without a matching config property', function () {
            var instance = {
                config: {
                    myOtherProp: 22
                }
            };

            expect(instance).not.toHaveConfig('myProp');
        });
    });

    describe('toHaveBoundEvent', function () {
        it('should match and pass a string binding against an expected string', function () {
            var addListener = jasmine.createSpy('addListener');

            addListener({
                click: 'onSomethingClick'
            });

            expect(addListener).toHaveBoundEvent('click', 'onSomethingClick');
        });

        it('should match and pass given a spy created with jasmine.createSpyObject', function () {
            var someObject = jasmine.createSpyObj('application', ['on']);

            function onSomeEvent() { };

            someObject.on({
                'someEvent': onSomeEvent
            });

            expect(someObject.on).toHaveBoundEvent('someEvent', onSomeEvent);
        });
    });

    describe('toHaveControlledEvent', function () {
        var control;

        beforeEach(function () {
            control = jasmine.createSpy('control');
        });

        it('should match and pass a control binding with selector config', function () {
            function onSomeClick() { }

            control({
                '#some-selector': {
                    click: onSomeClick
                }
            });

            expect(control).toHaveControlledEvent('click', onSomeClick, '#some-selector');
        });

        it('should match and pass a control binding with selector argument', function () {
            function onSomeClick() { }

            control('#some-selector', {
                click: onSomeClick
            });

            expect(control).toHaveControlledEvent('click', onSomeClick, '#some-selector');
        });

        it('should match and pass a control binding with selector and call arguments', function () {
            function onSomeClick() { }

            control('#some-selector1', {
                click: onSomeClick
            });

            control('#some-selector2', {
                click: onSomeClick
            });

            expect(control).toHaveControlledEvent('click', onSomeClick, '#some-selector2', 1);
        });
    });

    describe('toHaveBeenCalledWithConfig', function () {
        var spyWithConfig,
            config = {
                firstKey: true,
                secondKey: 2,
                thirdKey: 'three',
                fourthKey: [1, 2, 3, 4]
            };

        beforeEach(function () {
            spyWithConfig = jasmine.createSpy('spyWithConfig');
        });

        it('should pass given a first argument config with two matching keys', function () {
            spyWithConfig(config);

            expect(spyWithConfig).toHaveBeenCalledWithConfig({
                firstKey: true,
                thirdKey: 'three'
            });
        });

        it('should pass given a second argument config with three matching keys', function () {
            spyWithConfig('some name', config);

            expect(spyWithConfig).toHaveBeenCalledWithConfig({
                firstKey: true,
                secondKey: 2,
                thirdKey: 'three'
            }, 1);
        });

        it('should fail given a first argument config with non matching key', function () {
            spyWithConfig(config);

            expect(spyWithConfig).not.toHaveBeenCalledWithConfig({
                fifthKey: 555.555
            });
        });
    });
});